@@ -14,7 +14,7 @@ __all__ = [
 class AsyncSearchModelBinding(object):
 
     @database_sync_to_async
-    def search(self, *args, **kwargs):
+    def sync_search(self, *args, **kwargs):
         qs = self.get_queryset(*args, **kwargs)
         qs = self.filter_queryset(qs, *args, **kwargs)
         qs = self.paginate(qs, *args, **kwargs)
@@ -23,8 +23,8 @@ class AsyncSearchModelBinding(object):
         return data
 
     @bind('search')
-    async def async_search(self, *args, **kwargs):
-        await self.send('search', await self.search(*args, **kwargs))
+    async def search(self, *args, **kwargs):
+        await self.send('search', await self.sync_search(*args, **kwargs))
 
     def search_extra_data(self, *args, **kwargs):
         return {}
@@ -33,15 +33,15 @@ class AsyncSearchModelBinding(object):
 class AsyncRetrieveModelBinding(object):
 
     @database_sync_to_async
-    def retrieve(self, *args, **kwargs):
+    def sync_retrieve(self, *args, **kwargs):
         instance = self.get_object(*args, **kwargs)
         data = self.serialize(instance, *args, **kwargs)
         data.update(self.retrieve_extra_data(instance, *args, **kwargs))
         return data
 
     @bind('retrieve')
-    async def async_retrieve(self, *args, **kwargs):
-        await self.send('retrieve', await self.retrieve(*args, **kwargs))
+    async def retrieve(self, *args, **kwargs):
+        await self.send('retrieve', await self.sync_retrieve(*args, **kwargs))
 
     def retrieve_extra_data(self, data, *args, **kwargs):
         return {}
@@ -52,30 +52,31 @@ class AsyncSaveModelBinding(object):
     @database_sync_to_async
     def save(self, *args, **kwargs):
         form = self.get_form(*args, **kwargs)
-        if form.is_valid():
+        if self.form_is_valid(form):
             self.save_form(form, *args, **kwargs)
-            form.instance.update()
             return True
         else:
             return form.errors
-        return instance_data
+
+    def form_is_valid(self, form):
+        return form.is_valid()
+
+    def get_form_fields(self):
+        return self.fields
 
     def save_form(self, form, *args, **kwargs):
-        form.save()
+        return form.save()
 
     def get_form(self, data):
         instance = self.get_object(data)
         if not self.model_form:
-            form = modelform_factory(Product, fields=self.fields)(data, instance=instance)
+            form = modelform_factory(Product, fields=self.get_form_fields())(data, instance=instance)
         else:
             form = model_form()
 
     @bind('save')
     async def async_save(self, *args, **kwargs):
-        instance = await database_sync_to_async(self.get_object)(*args, **kwargs)
-        instance_data = await database_sync_to_async(self.serialize)(instance, *args, **kwargs)
-        instance_data.update(await database_sync_to_async(self.retrieve_extra_data)(instance, *args, **kwargs))
-        await self.send('retrieve', instance_data)
+        await self.send('save', await self.save(*args, **kwargs))
 
     def save_extra_data(self, *args, **kwargs):
         return {}
